# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from .download import download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_file(xz_url + "Baga Rain Shader_3.4.blend",down_path)
download_file(xz_url + "Baga_Rain_Generator_NodeTree_V10.blend",down_path)
print(down_path.name)


bl_info = {
    "name" : "Baga Rain Generator",
    "author" : "Antoine Bagattini",
    "description" : "Procedural Rain Generator based on Geometry Nodes",
    "blender" : (4, 0, 0),
    "version" : (1, 0, 9),
    "location" : "",
    "warning" : "",
    "category" : "Generic"
}

import bpy

from . rain_ui import RAIN_PT_Panel, RAIN_tooltips
from . rain_add import RAIN_OP_add
from . rain_ui_op import RAIN_OP_switchinput, RAIN_ADD_Target, RAIN_REMOVE_Target
from . rain_remove import RAIN_DELETE_Rain
from . rain_add_shader import RAIN_OP_add_shader, RAIN_UL_shader_list
from . import rain_icons

#############
# PREFERENCES
#############

class RAIN_Preferences(bpy.types.AddonPreferences):
    bl_idname = __name__
    
    # UI PROP
    rain_main_settings: bpy.props.BoolProperty(name="Main Settings", default=False)
    rain_target_settings: bpy.props.BoolProperty(name="Main Settings", default=False)
    rain_droplets_settings: bpy.props.BoolProperty(name="Main Settings", default=False)
    rain_splashes_settings: bpy.props.BoolProperty(name="Main Settings", default=False)
    rain_wind_settings: bpy.props.BoolProperty(name="Main Settings", default=False)

    # PREFERENCES PROP
    rain_preferences: bpy.props.BoolProperty(name="Rain Generator Preferences", default=False)
    support: bpy.props.BoolProperty(name="Rain Generator Preferences", default=False)
    our_addon: bpy.props.BoolProperty(name="Rain Generator Preferences", default=False)

    rain_distribution_method: bpy.props.BoolProperty(name="Distribution Method", default=False)

    splash_density_viewport: bpy.props.IntProperty(name="Density Viewport", default=100)
    splash_density_render: bpy.props.IntProperty(name="Density Render", default=1000)

    splash_radius_viewport: bpy.props.FloatProperty(name="Radius Viewport", default=0.002, precision=4)
    splash_radius_render: bpy.props.FloatProperty(name="Radius Render", default=0.002, precision=4)

    rain_density: bpy.props.FloatProperty(name="Density", default=5)
    rain_gravity: bpy.props.FloatProperty(name="Gravity", default=1.0)
    rain_altitude: bpy.props.FloatProperty(name="Altitude", default=30.0)
    rain_speed: bpy.props.FloatProperty(name="Droplets Speed", default=20.0)
    splash_speed: bpy.props.FloatProperty(name="Splashes Speed", default=35.0)
    
    droplet_scale: bpy.props.FloatVectorProperty(name="Droplet Scale", default=(0.1, 0.1, 0.2))
    pers_bouncing: bpy.props.FloatProperty(name="Bouncing Perpendicular", default=0.25)
    copl_bouncing: bpy.props.FloatProperty(name="Bouncing Coplanar", default=0.2)
    
    randomize_impact: bpy.props.FloatProperty(name="Randomize Impact", default=0.1)
    splash_shape: bpy.props.FloatProperty(name="Splashes Shape", default=0.5)

    def draw(self, context):
        layout = self.layout
        
        ###################################################################################
        # PREFERENCES
        ###################################################################################
        box = layout.box()
        box.prop(self, 'rain_preferences', text = "Rain Preferences", emboss = False, icon = "SETTINGS")
        if self.rain_preferences == True:
            col = box.column(align=True)
            col.label(text="Distribution Method :")
            row = col.row(align=True)
            row.prop(self, 'rain_distribution_method', invert_checkbox=True, text = "Accurate", toggle=True)
            row.prop(self, 'rain_distribution_method', text = "Large Scene", toggle=True)
            col.label(text="Rain default values :")
            col.prop(self, 'rain_density', text = "Density")
            col.prop(self, 'rain_gravity', text = "Gravity")
            col.prop(self, 'rain_altitude')
            col.prop(self, 'rain_speed', text = "Droplets Speed")
            col.prop(self, 'splash_speed', text = "Splashes Speed")
            row= col.row(align=True)
            row.prop(self, 'droplet_scale', text = "Droplets Scale")
            
            col.label(text="Splashes :")
            row = col.row()
            rcol = row.column(align = True)
            rcol.prop(self, 'splash_density_viewport')
            rcol.prop(self, 'splash_density_render')

            rcol = row.column(align = True)
            rcol.prop(self, 'splash_radius_viewport')
            rcol.prop(self, 'splash_radius_render')
            
            col.label(text="Bouncing")
            col.prop(self, 'pers_bouncing')
            col.prop(self, 'copl_bouncing')
            
            col.label(text="Impact")
            col.prop(self, 'randomize_impact')
            col.prop(self, 'splash_shape')

            col.label(text="Don't forget to save preferences !", icon ='ERROR')


            
        ###################################################################################
        # HELP & SUPPORT
        ###################################################################################
        box = layout.box()
        box.prop(self, 'support', text = "Support", emboss = False, icon = "HELP")
        if self.support == True:
            box.label(text="You can also contact us via BlenderMarket/Gumroad (see the product page)")
            row = box.row(align=True)
            row.scale_y = 1.5
            row.operator("wm.url_open", text="Discord !", icon = 'VIEW_PAN').url = "https://discord.gg/b7wg9rSKc7"


        ###################################################################################
        # OUR ADDONS
        ###################################################################################
        box = layout.box()
        box.prop(self, 'our_addon', text = "Our Addons !", emboss = False, icon = "FUND")
        if self.our_addon == True:
            col = box.column(align=True)
            row = col.row(align=True)
            row.scale_y = 1.5
            row.operator("wm.url_open", text="BagaPie Assets - [600 Assets]", icon = 'ASSET_MANAGER').url = "https://abaga.gumroad.com/l/GcYmPC"
            row.scale_x = 2
            row.operator("wm.url_open", text="", icon = 'PLAY').url = "https://youtu.be/uK7t_qDjm_0?t=50"

            row = box.row(align=True)
            row.scale_y = 1.5
            row.operator("wm.url_open", text="Baga Ivy Generator", icon = 'FUND').url = "https://blendermarket.com/products/bagaivy"
            row.scale_x = 2
            row.operator("wm.url_open", text="", icon = 'PLAY').url = "https://youtu.be/R6lt1QzV8jg"

            row = box.row(align=True)
            row.scale_y = 1.5
            row.operator("wm.url_open", text="Baga Rain Generator", icon = 'MOD_FLUIDSIM').url = "https://blendermarket.com/products/rain-generator"
            row.scale_x = 2
            row.operator("wm.url_open", text="", icon = 'PLAY').url = "https://abaga.gumroad.com/l/rain"

            row = box.row(align=True)
            row.scale_y = 1.5
            row.operator("wm.url_open", text="Quick Compo", icon = 'NODE_COMPOSITING').url = "https://abaga.gumroad.com/l/QCompo"
            row.scale_x = 2
            row.operator("wm.url_open", text="", icon = 'PLAY').url = "https://youtu.be/ZGN9YxvqXgM"

            col = box.column(align=True)
            col.label(text="Generators / Little Addons / Files :")
            row = col.row(align=True)
            row.scale_x = 1.2
            row.operator("wm.url_open", text="Symbiote Generator").url = "https://abaga.gumroad.com/l/SkyIVq"
            row.operator("wm.url_open", text="Blender and Print SpongeBob").url = "https://laura3dcraft.gumroad.com/l/bcvfa"
            row = col.row(align=True)
            row.scale_x = 1.2
            row.operator("wm.url_open", text="Ivy Generator").url = "https://abaga.gumroad.com/l/ivygen"
            row.operator("wm.url_open", text="Arch Generator").url = "https://abaga.gumroad.com/l/UlIvj"
            row.operator("wm.url_open", text="BagaPassesSaver").url = "https://abaga.gumroad.com/l/MQcAd"
            row = col.row(align=True)
            row.scale_x = 1.2
            row.operator("wm.url_open", text="Render Device Switcher").url = "https://abaga.gumroad.com/l/AKNdXX"
            row.operator("wm.url_open", text="Lego Generator").url = "https://abaga.gumroad.com/l/zlcrs"
            row.operator("wm.url_open", text="Fantasy Gate Generator").url = "https://abaga.gumroad.com/l/hcvvq"


classes = [
    RAIN_Preferences,
    RAIN_PT_Panel,
    RAIN_OP_add,
    RAIN_OP_switchinput,
    RAIN_ADD_Target,
    RAIN_REMOVE_Target,
    RAIN_DELETE_Rain,
    RAIN_OP_add_shader,
    RAIN_tooltips,
    RAIN_UL_shader_list,
    ]


def register():    
    rain_icons.import_icons()

    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():    
    rain_icons.unload_icons()

    for cls in classes:
        bpy.utils.unregister_class(cls)
    
if __name__ == "__main__":
    register()